@@ -1,25 +1,12 @@
-// File_Tiff - Info for TIFF files
-// Copyright (C) 2005-2012 MediaArea.net SARL, Info@MediaArea.net
-//
-// This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Library General Public License as published by
-// the Free Software Foundation, either version 2 of the License, or
-// any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public License
-// along with this library. If not, see <http://www.gnu.org/licenses/>.
-//
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
+ *
+ *  Use of this source code is governed by a BSD-style license that can
+ *  be found in the License.html file in the root of the source tree.
+ */
 
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-// TIFF - Format
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+//
+// TIFF Format
 //
 // From
 // http://partners.adobe.com/public/developer/en/tiff/TIFF6.pdf
@@ -66,12 +53,23 @@ namespace Tiff_Tag
     const int16u Compression                = 259;
     const int16u PhotometricInterpretation  = 262;
     const int16u ImageDescription           = 270;
+    const int16u Make                       = 271;
+    const int16u Model                      = 272;
+    const int16u StripOffsets               = 273;
     const int16u SamplesPerPixel            = 277;
+    const int16u RowsPerStrip               = 278;
+    const int16u StripByteCounts            = 279;
+    const int16u XResolution                = 282;
+    const int16u YResolution                = 283;
+    const int16u PlanarConfiguration        = 284;
+    const int16u ResolutionUnit             = 296;
+    const int16u Software                   = 305;
+    const int16u DateTime                   = 306;
     const int16u ExtraSamples               = 338;
 }
 
 //---------------------------------------------------------------------------
-const char* Tiff_Tag_Name(int32u Tag)
+static const char* Tiff_Tag_Name(int32u Tag)
 {
     switch (Tag)
     {
@@ -81,9 +79,20 @@ const char* Tiff_Tag_Name(int32u Tag)
         case Tiff_Tag::Compression                  : return "Compression";
         case Tiff_Tag::PhotometricInterpretation    : return "PhotometricInterpretation";
         case Tiff_Tag::ImageDescription             : return "ImageDescription";
+        case Tiff_Tag::Make                         : return "Make";
+        case Tiff_Tag::Model                        : return "Model";
+        case Tiff_Tag::StripOffsets                 : return "StripOffsets";
         case Tiff_Tag::SamplesPerPixel              : return "SamplesPerPixel";
+        case Tiff_Tag::RowsPerStrip                 : return "RowsPerStrip";
+        case Tiff_Tag::StripByteCounts              : return "StripByteCounts";
+        case Tiff_Tag::XResolution                  : return "XResolution";
+        case Tiff_Tag::YResolution                  : return "YResolution";
+        case Tiff_Tag::PlanarConfiguration          : return "PlanarConfiguration";
+        case Tiff_Tag::ResolutionUnit               : return "ResolutionUnit";
+        case Tiff_Tag::Software                     : return "Software";
+        case Tiff_Tag::DateTime                     : return "DateTime";
         case Tiff_Tag::ExtraSamples                 : return "ExtraSamples";
-        default                                     : return "Unknown";
+        default                                     : return "";
     }
 }
 
@@ -91,36 +100,42 @@ const char* Tiff_Tag_Name(int32u Tag)
 namespace Tiff_Type
 {
     const int16u Byte       = 1;
+    const int16u ASCII      = 2;
     const int16u Short      = 3;
     const int16u Long       = 4;
+    const int16u Rational   = 5;
 }
 
 //---------------------------------------------------------------------------
-const char* Tiff_Type_Name(int32u Type)
+static const char* Tiff_Type_Name(int32u Type)
 {
     switch (Type)
     {
         case Tiff_Type::Byte                        : return "Byte";
+        case Tiff_Type::ASCII                       : return "ASCII";
         case Tiff_Type::Short                       : return "Short";
         case Tiff_Type::Long                        : return "Long";
+        case Tiff_Type::Rational                    : return "Rational";
         default                                     : return ""; //Unknown
     }
 }
 
 //---------------------------------------------------------------------------
-const int8u Tiff_Type_Size(int32u Type)
+static const int8u Tiff_Type_Size(int32u Type)
 {
     switch (Type)
     {
         case Tiff_Type::Byte                        : return 1;
+        case Tiff_Type::ASCII                       : return 1;
         case Tiff_Type::Short                       : return 2;
         case Tiff_Type::Long                        : return 4;
+        case Tiff_Type::Rational                    : return 8;
         default                                     : return 0; //Unknown
     }
 }
 
 //---------------------------------------------------------------------------
-const char* Tiff_Compression(int32u Compression)
+static const char* Tiff_Compression(int32u Compression)
 {
     switch (Compression)
     {
@@ -135,7 +150,7 @@ const char* Tiff_Compression(int32u Compression)
 }
 
 //---------------------------------------------------------------------------
-const char* Tiff_Compression_Mode(int32u Compression)
+static const char* Tiff_Compression_Mode(int32u Compression)
 {
     switch (Compression)
     {
@@ -149,7 +164,7 @@ const char* Tiff_Compression_Mode(int32u Compression)
 }
 
 //---------------------------------------------------------------------------
-const char* Tiff_PhotometricInterpretation(int32u PhotometricInterpretation)
+static const char* Tiff_PhotometricInterpretation(int32u PhotometricInterpretation)
 {
     switch (PhotometricInterpretation)
     {
@@ -166,7 +181,7 @@ const char* Tiff_PhotometricInterpretation(int32u PhotometricInterpretation)
 }
 
 //---------------------------------------------------------------------------
-const char* Tiff_PhotometricInterpretation_ColorSpace (int32u PhotometricInterpretation)
+static const char* Tiff_PhotometricInterpretation_ColorSpace (int32u PhotometricInterpretation)
 {
     switch (PhotometricInterpretation)
     {
@@ -182,18 +197,7 @@ const char* Tiff_PhotometricInterpretation_ColorSpace (int32u PhotometricInterpr
     }
 }
 
-const char* Tiff_ExtraSamples(int32u ExtraSamples)
-{
-    switch (ExtraSamples)
-    {
-        case     0 :
-        case     1 : return "Associated Alpha";
-        case     2 : return "Unassocieted Alpha";
-        default    : return ""; //Unknown
-    }
-}
-
-const char* Tiff_ExtraSamples_ColorSpace(int32u ExtraSamples)
+static const char* Tiff_ExtraSamples_ColorSpace(int32u ExtraSamples)
 {
     switch (ExtraSamples)
     {
@@ -242,13 +246,13 @@ bool File_Tiff::FileHeader_Begin()
 void File_Tiff::FileHeader_Parse()
 {
     //The only IFD that is known at forehand is the first one, it's offset is placed byte 4-7 in the file.
-    int32u IFDOffset;
+    int32u FirstIFDOffset;
     Skip_B4(                                                    "Magic");
-    Get_X4 (IFDOffset,                                          "IFDOffset");
+    Get_X4 (FirstIFDOffset,                                     "FirstIFDOffset");
 
     FILLING_BEGIN();
         //Initial IFD
-        GoTo(IFDOffset, "TIFF");
+        GoTo_IfNeeded(FirstIFDOffset);
     FILLING_END();
 }
 
@@ -266,7 +270,15 @@ void File_Tiff::Header_Parse()
             IfdItems.clear(); //There was a problem during the seek, trashing remaining positions from last IFD
         else
         {
-            Header_Fill_Code(IfdItems.begin()->second.Tag, Tiff_Tag_Name(IfdItems.begin()->second.Tag));
+            #ifdef MEDIAINFO_TRACE
+                const char* Name=Tiff_Tag_Name(IfdItems.begin()->second.Tag);
+                if (!Name[0]) //Unknown
+                    Header_Fill_Code(IfdItems.begin()->second.Tag, Ztring::ToZtring(IfdItems.begin()->second.Tag));
+                else
+                    Header_Fill_Code(IfdItems.begin()->second.Tag, Name);
+            #else //MEDIAINFO_TRACE
+                Header_Fill_Code(IfdItems.begin()->second.Tag);
+            #endif //MEDIAINFO_TRACE
             Header_Fill_Size(Tiff_Type_Size(IfdItems.begin()->second.Type)*IfdItems.begin()->second.Count);
             return;
         }
@@ -306,7 +318,7 @@ void File_Tiff::Data_Parse()
         Infos[Tiff_Tag::BitsPerSample]=__T("1");
 
         //Parsing new IFD
-        while (Element_Offset+4<Element_Size)
+        while (Element_Offset+8+4<Element_Size)
             Read_Directory();
         Get_X4 (IFDOffset,                                          "IFDOffset");
     }
@@ -319,15 +331,18 @@ void File_Tiff::Data_Parse()
 
     //Some items are not inside the directory, jumping to the offset
     if (!IfdItems.empty())
-        GoTo(IfdItems.begin()->first, "TIFF");
+        GoTo_IfNeeded(IfdItems.begin()->first);
     else
     {
         //This IFD is finished, filling data then going to next IFD
         Data_Parse_Fill();
         if (IFDOffset)
-            GoTo(IFDOffset, "TIFF");
+            GoTo_IfNeeded(IFDOffset);
         else
+        {
             Finish(); //No more IFDs
+            GoToFromEnd(0);
+        }
     }
 }
 
@@ -335,6 +350,8 @@ void File_Tiff::Data_Parse()
 void File_Tiff::Data_Parse_Fill()
 {
     Stream_Prepare(Stream_Image);
+    Fill(Stream_Image, 0, Image_Format_Settings, LittleEndian?"Little":"Big");
+    Fill(Stream_Image, 0, Image_Format_Settings_Endianness, LittleEndian?"Little":"Big");
 
     infos::iterator Info;
 
@@ -351,7 +368,19 @@ void File_Tiff::Data_Parse_Fill()
     //BitsPerSample
     Info=Infos.find(Tiff_Tag::BitsPerSample);
     if (Info!=Infos.end())
+    {
+        if (Info->second.size()>1)
+        {
+            bool IsOk=true;
+            for (size_t Pos=1; Pos<Info->second.size(); ++Pos)
+                if (Info->second[Pos]!=Info->second[0])
+                    IsOk=false;
+            if (IsOk)
+                Info->second.resize(1); //They are all same, we display 1 piece of information
+        }
+
         Fill(Stream_Image, StreamPos_Last, Image_BitDepth, Info->second.Read());
+    }
 
     //Compression
     Info=Infos.find(Tiff_Tag::Compression);
@@ -377,6 +406,83 @@ void File_Tiff::Data_Parse_Fill()
     if (Info!=Infos.end())
         Fill(Stream_Image, StreamPos_Last, Image_Title, Info->second.Read());
 
+    //Make
+    Info=Infos.find(Tiff_Tag::Make);
+    if (Info!=Infos.end())
+        Fill(Stream_General, StreamPos_Last, General_Encoded_Application_CompanyName, Info->second.Read());
+
+    //Model
+    Info=Infos.find(Tiff_Tag::Model);
+    if (Info!=Infos.end())
+        Fill(Stream_General, StreamPos_Last, General_Encoded_Library_Name, Info->second.Read());
+
+    //XResolution
+    Info=Infos.find(Tiff_Tag::XResolution);
+    if (Info!=Infos.end())
+    {
+        Fill(Stream_Image, StreamPos_Last, "Density_X", Info->second.Read());
+        Fill_SetOptions(Stream_Image, StreamPos_Last, "Density_X", "N NT");
+    }
+
+    //YResolution
+    Info=Infos.find(Tiff_Tag::XResolution);
+    if (Info!=Infos.end())
+    {
+        Fill(Stream_Image, StreamPos_Last, "Density_Y", Info->second.Read());
+        Fill_SetOptions(Stream_Image, StreamPos_Last, "Density_Y", "N NT");
+    }
+
+    //ResolutionUnit
+    Info=Infos.find(Tiff_Tag::ResolutionUnit);
+    if (Info!=Infos.end())
+    {
+        switch (Info->second.Read().To_int32u())
+        {
+            case 0 : break;
+            case 1 : Fill(Stream_Image, StreamPos_Last, "Density_Unit", "dpcm"); Fill_SetOptions(Stream_Image, StreamPos_Last, "Density_Unit", "N NT"); break;
+            case 2 : Fill(Stream_Image, StreamPos_Last, "Density_Unit", "dpi"); Fill_SetOptions(Stream_Image, StreamPos_Last, "Density_Unit", "N NT"); break;
+            default: Fill(Stream_Image, StreamPos_Last, "Density_Unit", Info->second.Read()); Fill_SetOptions(Stream_Image, StreamPos_Last, "Density_Unit", "N NT");
+        }
+    }
+    else if (Infos.find(Tiff_Tag::XResolution)!=Infos.end() || Infos.find(Tiff_Tag::XResolution)!=Infos.end())
+    {
+        Fill(Stream_Image, StreamPos_Last, "Density_Unit", "dpi");
+        Fill_SetOptions(Stream_Image, StreamPos_Last, "Density_Unit", "N NT");
+    }
+    
+    //XResolution or YResolution 
+    if (Infos.find(Tiff_Tag::XResolution)!=Infos.end() || Infos.find(Tiff_Tag::XResolution)!=Infos.end())
+    {
+        Ztring X=Retrieve(Stream_Image, StreamPos_Last, "Density_X");
+        if (X.empty())
+            X.assign(1, __T('?'));
+        Ztring Y=Retrieve(Stream_Image, StreamPos_Last, "Density_Y");
+        if (Y.empty())
+            Y.assign(1, __T('?'));
+        if (X!=Y)
+        {
+            X+=__T('x');
+            X+=Y;
+        }
+        Y=Retrieve(Stream_Image, StreamPos_Last, "Density_Unit");
+        if (!Y.empty())
+        {
+            X+=__T(' ');
+            X+=Y;
+            Fill(Stream_Image, StreamPos_Last, "Density/String", X);
+        }
+    }
+
+    //Software
+    Info=Infos.find(Tiff_Tag::Software);
+    if (Info!=Infos.end())
+        Fill(Stream_General, StreamPos_Last, General_Encoded_Application_Name, Info->second.Read());
+
+    //DateTime
+    Info=Infos.find(Tiff_Tag::DateTime);
+    if (Info!=Infos.end())
+        Fill(Stream_Image, StreamPos_Last, Image_Encoded_Date, Info->second.Read());
+
     //ExtraSamples
     Info=Infos.find(Tiff_Tag::ExtraSamples);
     if (Info!=Infos.end())
@@ -401,15 +507,22 @@ void File_Tiff::Read_Directory()
     Get_X2 (IfdItem.Tag,                                        "Tag"); Param_Info1(Tiff_Tag_Name(IfdItem.Tag));
     Get_X2 (IfdItem.Type,                                       "Type"); Param_Info1(Tiff_Type_Name(IfdItem.Type));
     Get_X4 (IfdItem.Count,                                      "Count");
-    Element_Name(Tiff_Tag_Name(IfdItem.Tag));
+    #ifdef MEDIAINFO_TRACE
+        const char* Name=Tiff_Tag_Name(IfdItem.Tag);
+        if (!Name[0]) //Unknown
+            Element_Name(Ztring::ToZtring(IfdItem.Tag));
+        else
+            Element_Name(Name);
+    #endif //MEDIAINFO_TRACE
 
-    if (Tiff_Type_Size(IfdItem.Type)*IfdItem.Count<=4)
+    int32u Size=Tiff_Type_Size(IfdItem.Type)*IfdItem.Count;
+    if (Size<=4)
     {
         GetValueOffsetu(IfdItem);
 
-        /* Padding up, skip dummy bytes */
-        if (Tiff_Type_Size(IfdItem.Type)*IfdItem.Count<4)
-            Skip_XX(Tiff_Type_Size(IfdItem.Type)*IfdItem.Count, "Padding");
+        //Padding up, skip dummy bytes
+        if (Size<4)
+            Skip_XX(4-Size,                                     "Padding");
     }
     else
     {
@@ -446,13 +559,11 @@ void File_Tiff::Get_X4(int32u &Info, const char* Name)
 void File_Tiff::GetValueOffsetu(ifditem &IfdItem)
 {
     ZtringList &Info=Infos[IfdItem.Tag]; Info.clear(); Info.Separator_Set(0, __T(" / "));
-    const char* Name=Tiff_Tag_Name(IfdItem.Tag);
 
-    if (IfdItem.Count>=10)
+    if (IfdItem.Type!=Tiff_Type::ASCII && IfdItem.Count>=1000)
     {
         //Too many data, we don't currently need it and we skip it
-        Skip_XX(Element_Size,                       Name);
-        Info.clear();
+        Skip_XX(Tiff_Type_Size(IfdItem.Type)*IfdItem.Count,     "Data");
         return;
     }
 
@@ -463,10 +574,7 @@ void File_Tiff::GetValueOffsetu(ifditem &IfdItem)
                 {
                     int8u Ret8;
                     #if MEDIAINFO_TRACE
-                        if (LittleEndian)
-                            Get_L1 (Ret8,                       Name);
-                        else
-                            Get_B1 (Ret8,                       Name);
+                            Get_B1 (Ret8,                       "Data"); //L1 and B1 are same
                         Element_Info1(Ztring::ToZtring(Ret8));
                     #else //MEDIAINFO_TRACE
                         if (Element_Offset+1>Element_Size)
@@ -474,25 +582,28 @@ void File_Tiff::GetValueOffsetu(ifditem &IfdItem)
                             Trusted_IsNot();
                             break;
                         }
-                        if (LittleEndian)
-                            Ret8=LittleEndian2int8u(Buffer+Buffer_Offset+(size_t)Element_Offset);
-                        else
-                            Ret8=BigEndian2int8u(Buffer+Buffer_Offset+(size_t)Element_Offset);
+                        Ret8=BigEndian2int8u(Buffer+Buffer_Offset+(size_t)Element_Offset); //LittleEndian2int8u and BigEndian2int8u are same
                         Element_Offset++;
                     #endif //MEDIAINFO_TRACE
                     Info.push_back(Ztring::ToZtring(Ret8));
                 }
                 break;
-
+        case 2:                /* ASCII */
+                {
+                    string Data;
+                    Get_String(IfdItem.Count, Data,             "Data"); Element_Info1(Data.c_str()); //TODO: multiple strings separated by NULL
+                    Info.push_back(Ztring().From_UTF8(Data.c_str()));
+                }
+                break;
         case 3:                /* 16-bit (2-byte) unsigned integer. */
                 for (int16u Pos=0; Pos<IfdItem.Count; Pos++)
                 {
                     int16u Ret16;
                     #if MEDIAINFO_TRACE
                         if (LittleEndian)
-                            Get_L2 (Ret16,                      Name);
+                            Get_L2 (Ret16,                      "Data");
                         else
-                            Get_B2 (Ret16,                      Name);
+                            Get_B2 (Ret16,                      "Data");
                         switch (IfdItem.Tag)
                         {
                             case Tiff_Tag::Compression : Element_Info1(Tiff_Compression(Ret16)); break;
@@ -521,9 +632,9 @@ void File_Tiff::GetValueOffsetu(ifditem &IfdItem)
                     int32u Ret32;
                     #if MEDIAINFO_TRACE
                         if (LittleEndian)
-                            Get_L4 (Ret32,                      Name);
+                            Get_L4 (Ret32,                      "Data");
                         else
-                            Get_B4 (Ret32,                      Name);
+                            Get_B4 (Ret32,                      "Data");
                         Element_Info1(Ztring::ToZtring(Ret32));
                     #else //MEDIAINFO_TRACE
                         if (Element_Offset+4>Element_Size)
@@ -532,26 +643,68 @@ void File_Tiff::GetValueOffsetu(ifditem &IfdItem)
                             break;
                         }
                         if (LittleEndian)
-                            Ret32=LittleEndian2int16u(Buffer+Buffer_Offset+(size_t)Element_Offset);
+                            Ret32=LittleEndian2int32u(Buffer+Buffer_Offset+(size_t)Element_Offset);
                         else
-                            Ret32=BigEndian2int16u(Buffer+Buffer_Offset+(size_t)Element_Offset);
+                            Ret32=BigEndian2int32u(Buffer+Buffer_Offset+(size_t)Element_Offset);
                         Element_Offset+=4;
                     #endif //MEDIAINFO_TRACE
                     Info.push_back(Ztring::ToZtring(Ret32));
                 }
                 break;
 
-        default:            //Unknown
+        case 5:                /* 2x32-bit (2x4-byte) unsigned integers */
+                for (int16u Pos=0; Pos<IfdItem.Count; Pos++)
                 {
-                    if (LittleEndian)
-                        Skip_L4(                                Name);
-                    else
-                        Skip_B4(                                Name);
-                    Info.clear(); //We actually do not know the type
+                    int32u N, D;
+                    #if MEDIAINFO_TRACE
+                        if (LittleEndian)
+                        {
+                            Get_L4 (N,                          "Numerator");
+                            Get_L4 (D,                          "Denominator");
+                        }
+                        else
+                        {
+                            Get_B4 (N,                          "Numerator");
+                            Get_B4 (D,                          "Denominator");
+                        }
+                        Element_Info1(Ztring::ToZtring(((float64)N)/D));
+                    #else //MEDIAINFO_TRACE
+                        if (Element_Offset+8>Element_Size)
+                        {
+                            Trusted_IsNot();
+                            break;
+                        }
+                        if (LittleEndian)
+                        {
+                            N=LittleEndian2int32u(Buffer+Buffer_Offset+(size_t)Element_Offset);
+                            D=LittleEndian2int32u(Buffer+Buffer_Offset+(size_t)Element_Offset);
+                        }
+                        else
+                        {
+                            N=BigEndian2int32u(Buffer+Buffer_Offset+(size_t)Element_Offset);
+                            D=BigEndian2int32u(Buffer+Buffer_Offset+(size_t)Element_Offset);
+                        }
+                        Element_Offset+=8;
+                    #endif //MEDIAINFO_TRACE
+                    Info.push_back(Ztring::ToZtring(((float64)N)/D, D==1?0:3));
                 }
+                break;
+
+        default:            //Unknown
+                Skip_XX(Tiff_Type_Size(IfdItem.Type)*IfdItem.Count, "Data");
     }
 }
 
+//---------------------------------------------------------------------------
+void File_Tiff::GoTo_IfNeeded(int64u GoTo_) //TODO: move that in a generic section, but tests showed regressions, for later (main difference is text trace, info part)
+{
+    if (File_Offset+Buffer_Offset+Element_Offset==GoTo_)
+        return; //Useless
+
+    GoTo(GoTo_);
+}
+
+
 } //NameSpace
 
 #endif
